use bytes::{BufMut, Bytes, BytesMut};
use tiny_skia::{Color, Pixmap, PremultipliedColorU8};

use loupedeck_serial::util::Endianness;

use crate::runner::state::Key;

pub fn render_key(width: u16, height: u16, buffer_endianness: Endianness, state: Option<&Key>) -> Bytes {
    let mut canvas = Pixmap::new(width as u32, height as u32).unwrap();

    if let Some(state) = state {
        canvas.fill(if state.label.is_empty() {
            Color::WHITE
        } else {
            Color::from_rgba8(0, 255, 0, 255)
        });
    } else {
        canvas.fill(Color::BLACK);
    }

    convert_pixels_to_rgb565(canvas.pixels(), buffer_endianness).freeze()
}

fn convert_pixels_to_rgb565(pixels: &[PremultipliedColorU8], endianness: Endianness) -> BytesMut {
    let pixel_count = pixels.len();

    let mut result = BytesMut::with_capacity(pixel_count * 2);

    for pixel in pixels {
        let red = pixel.red() as u16;
        let green = pixel.green() as u16;
        let blue = pixel.blue() as u16;
        let color = ((red & 0b11111000) << 8) | ((green & 0b11111100) << 3) | (blue.wrapping_shr(3));

        match endianness {
            Endianness::LittleEndian => result.put_u16_le(color),
            Endianness::BigEndian => result.put_u16(color),
        }
    }

    result
}
